package pl.g00n.lmsimport.service;

import java.sql.SQLException;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import pl.g00n.lmsimport.domain.Invoice;

public class CsvParser {

	/***
	 * 
	 * @author Lukasz Czerepuk <lukasz@czerepuk.pl> 2012r.
	 * 
	 */

	/**
	 * Metoda ktora parsuje Stringa cvsowego na obiekty w Kolekcji
	 * 
	 * @param cvs
	 * @return
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	public ArrayList<Invoice> parse(String cvs) throws InstantiationException,
			IllegalAccessException, ClassNotFoundException, SQLException {

		// Logger
		Logger logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");

		// Tablice do splitowania
		String[] proFormaSplitArray = cvs.split("\"02\",");
		String[] targetSplitArray = new String[proFormaSplitArray.length - 1];

		// Baza sparsowanych i przerobionych na obiekty wplat
		ArrayList<Invoice> invoices = new ArrayList<Invoice>();

		// Tworzy liste csvowa przelewow, pomijajac naglowek
		for (int i = 1; i < proFormaSplitArray.length; i++) {
			targetSplitArray[i - 1] = proFormaSplitArray[i];
		}

		// Iteruje po csvowej liscie przelewow z pominietym naglowkiem,
		// tworzy obiekty Invoice i dodaje do bazy sparsowanych wplat
		for (String record : targetSplitArray) {
			String[] splitRecord = record.split("\",\"");

			Invoice invoice = new Invoice();
			invoice.setPrivateAccount(splitRecord[2]);
			invoice.setInvoiceNumber(splitRecord[3]);
			invoice.setSum(splitRecord[4]);
			invoice.setDate(splitRecord[5]);
			invoice.setDescription(splitRecord[8].replace("\"", ""));

			// Tworzymy id klienta, usuwajac prefix ktory jestem numerem konta
			invoice.setCustomerId(splitRecord[2].replaceFirst("^[0-9]{2}[0-9]{12}0+(?!$)", ""));

			// Jesli nie istnieje juz taka transakcja w bazie, dodajemy wplate
			// do bazy
			if (!new DbManager().isInvoiceCommited(invoice.getInvoiceNumber())) {
				invoices.add(invoice);
			} else {
				logger.warn("Istnieje juz wplata o numerze: "
						+ invoice.getInvoiceNumber());
			}
		}

		// Logujemy do log4ja liste platnosci ze sparsowanego emaila
		logger.info("Pobralem nastepujace platnosci: ");
		for (Invoice invoice : invoices) {
			logger.info(invoice.toString());

		}

		// Zwracamy baze sparsowanych wplat
		return invoices;
	}
}
