package pl.g00n.lmsimport.service;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import pl.g00n.lmsimport.domain.Invoice;

import com.mysql.jdbc.Connection;

public class DbManager {

	/***
	 * 
	 * @author Lukasz Czerepuk <lukasz@czerepuk.pl> 2012r.
	 * 
	 */

	private String hostname = "xxxx";
	private String username = "xxxx";
	private String password = "xxxxxx";
	Connection conn = null;

	public String getHostname() {
		return hostname;
	}

	public void setHostname(String hostname) {
		this.hostname = hostname;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public DbManager(String hostname, String username, String password) {
		super();
		this.hostname = hostname;
		this.username = username;
		this.password = password;
	}

	public DbManager() {

	}

	/**
	 * Metoda ktora utrwala w bazie LMSowej wplaty.
	 * 
	 * @param invoices
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 */
	public void persistInvoices(ArrayList<Invoice> invoices)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {

		// Logger
		Logger logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");

		// Inicjujemy polaczenia do bazy
		String url = "jdbc:mysql://" + this.hostname + "/lms";
		String urlLmsImport = "jdbc:mysql://" + this.hostname + "/lmsimport";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);
		Connection connLmsImport = (Connection) DriverManager.getConnection(
				urlLmsImport, this.username, this.password);

		logger.info("Dodaje wplaty do bazy...");

		String query = "INSERT INTO cashimport (date, value, customer, description, customerid) values "
				+ "(UNIX_TIMESTAMP(now()),?,?,?,?)";
		String queryLmsImport = "INSERT INTO invoice (invoice) value (?)";

		// Iterujemy po Liscie wplat, dodajemy je do bazy danych oraz do tabeli
		// ktora bedzie definiowac nam, ktore z wplat zostaly juz wprowadzone.
		for (Invoice invoice : invoices) {
			PreparedStatement ps = conn.prepareStatement(query);
			PreparedStatement psLmsImport = connLmsImport
					.prepareStatement(queryLmsImport);

			ps.setBigDecimal(1,
					new BigDecimal(invoice.getSum().replace(",", ".")));
			ps.setString(
					2,
					invoice.getCustomerId() + " | "
							+ invoice.getPrivateAccount());
			ps.setString(
					3,
					invoice.getInvoiceNumber() + ": "
							+ invoice.getDescription());
			ps.setInt(4, Integer.parseInt(invoice.getCustomerId()));

			psLmsImport.setString(1, invoice.getInvoiceNumber());

			ps.execute();

			psLmsImport.execute();

			logger.info(invoice.toString());
		}
		conn.close();
	}

	/**
	 * Metoda ktora sprawdza czy nazwa zalacznika ktory chcemy parsowac, nie
	 * zostal juz sparsowany wczesniej.
	 * 
	 * @param emailAttachmentName
	 * @return
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 */
	public boolean isAttachmentParsed(String emailAttachmentName)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {

		// Inicjujemy polaczenie do bazy
		String url = "jdbc:mysql://" + this.hostname + "/lmsimport";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);

		// Tworzymy zapytanie
		String query = "SELECT * from attachment WHERE name=?";

		// Tworzymy preparedStatement na podstawie powyzszego zapytania.
		// Uzupelniamy go wartosciami dla parametrow.
		PreparedStatement prest = conn.prepareStatement(query);
		prest.setString(1, emailAttachmentName);

		// Sprawdzamy czy istnieje jakikolwiek rekord
		// Jesli istnieje, zwracamy true, w przeciwnym wypadku false
		ResultSet rs = prest.executeQuery();
		if (rs.next()) {
			conn.close();
			return true;
		}

		conn.close();
		return false;

	}

	/**
	 * Metoda ktora zapisuje nazwy plikow zalacznikow ktore byly juz parsowane,
	 * aby uniknac podwojnego parsowania tego samego pliku.
	 * 
	 * @param attachmentName
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 */
	public void persistAttachment(String attachmentName)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {

		// Logger
		Logger logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");

		// Inicjujemy polaczenie do bazy
		String url = "jdbc:mysql://" + this.hostname + "/lmsimport";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);

		// Tworzymy zapytanie
		String query = "insert into attachment (name) values (?)";

		// Tworzymy preparedStatement na podstawie powyzszego zapytania.
		// Uzupelniamy go wartosciami dla parametrow.
		PreparedStatement preparedStmt = conn.prepareStatement(query);
		preparedStmt.setString(1, attachmentName);

		// Wykonujemy zapytanie
		preparedStmt.execute();

		logger.info("Dodaje sparsowany zalacznik o nazwie: " + attachmentName);
		// Zamykamy polaczenie.
		conn.close();

	}

	public boolean isInvoiceCommited(String invoiceNumber)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {

		// Inicjujemy polaczenie do bazy
		String url = "jdbc:mysql://" + this.hostname + "/lmsimport";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);

		// Tworzymy zapytanie
		String query = "SELECT * from invoice WHERE invoice=?";

		// Tworzymy preparedStatement na podstawie powyzszego zapytania.
		// Uzupelniamy go wartosciami dla parametrow.
		PreparedStatement prest = conn.prepareStatement(query);
		prest.setString(1, invoiceNumber);

		// Sprawdzamy czy istnieje jakikolwiek rekord
		// Jesli istnieje, zwracamy true, w przeciwnym wypadku false
		ResultSet rs = prest.executeQuery();
		if (rs.next()) {
			conn.close();
			return true;
		}

		conn.close();
		return false;
	}

	public void AcceptInvoices() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException, SQLException,
			IOException, InterruptedException {
		// Logger
		Logger logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");

		logger.info("Generuje raport wplat do akceptacji...");
		// Inicjujemy polaczenia do bazy
		String url = "jdbc:mysql://" + this.hostname + "/lms";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);
		// Tworzymy zapytanie
		String query = "SELECT id, customerid from cashimport where closed=0";

		Statement st = conn.createStatement();

		String formdata = "";
		ResultSet rs = st.executeQuery(query);
		while (rs.next()) {
			formdata += "customer[" + rs.getInt(1) + "]=" + rs.getInt(2)
					+ "&marks[" + rs.getInt(1) + "]=" + rs.getInt(1) + "&";
		}
		formdata += "allbox=Check All";
		logger.info("FormData do zaakceptowania: " + formdata);
		conn.close();

	}

	public ArrayList<Integer> getPeopleWhosMadeMistake()
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {
		// Inicjujemy polaczenie do bazy
		String url = "jdbc:mysql://" + this.hostname + "/lms";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);

		// Tworzymy zapytanie
		String query = "SELECT customerid from cash WHERE time > UNIX_TIMESTAMP(now())-86400 AND comment LIKE '%przelew%'";
		Statement st = conn.createStatement();
		ResultSet rs = st.executeQuery(query);
		ArrayList<Integer> clients = new ArrayList<Integer>();

		while (rs.next()) {
			clients.add(rs.getInt(1));
		}

		return clients;

	}

	public void makeBannerForPeopleWhosMadeMistake(ArrayList<Integer> clients)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {
		// Logger
		Logger logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");

		String url = "jdbc:mysql://" + this.hostname + "/pQdNetUsers";
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		conn = (Connection) DriverManager.getConnection(url, this.username,
				this.password);

		for (Integer client : clients) {
			String query = "SELECT ip FROM user WHERE lms_user_id=" + client;
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(query);
			while (rs.next()) {
				String bannerInsert = "INSERT INTO banner (ip, type, active, msg) values "
						+ "(?,?,?,?)";
				PreparedStatement ps = conn.prepareStatement(bannerInsert);
				ps.setString(1, rs.getString(1));
				ps.setInt(2, 6);
				ps.setInt(3, 1);
				ps.setString(
						4,
						"Dziękujemy za wpłatę jednak w przyszłości uprzejmie prosimy o wpłatę na nowy numer rachunku - szczegóły na ostatniej fakturze.\n Z góry dziękujemy");
				ps.execute();
				logger.info("Dodano banner dla ip: " + rs.getString(1));

			}
		}
		conn.close();
	}

}
