package pl.g00n.lmsimport.service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Properties;

import javax.mail.BodyPart;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMessage.RecipientType;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import pl.g00n.lmsimport.domain.Invoice;

/***
 * 
 * @author Lukasz Czerepuk <lukasz@czerepuk.pl> 2012r.
 * 
 */

public class EmailParser {

	// Pola
	private Logger logger;
	private String host = "";
	private String user = "";
	private String password = "";

	// Gettery oraz Settery
	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getUser() {
		return user;
	}

	public void setUser(String user) {
		this.user = user;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	// Konstruktory
	public EmailParser(String host, String user, String password) {
		super();
		this.host = host;
		this.user = user;
		this.password = password;

		// Log4j
		logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");
	}

	public EmailParser() {

		// Log4j
		logger = Logger.getLogger(EmailParser.class);
		PropertyConfigurator.configure("resources/log4j.properties");
	}

	/**
	 * Metoda ktora laczy sie skrzynki pocztowej, pobiera najnowszego cvsa i
	 * zwraca go.
	 * 
	 * @return
	 * @throws MessagingException
	 * @throws IOException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 */
	public String getContentFromMessage(int messageFromEnd)
			throws MessagingException, IOException, InstantiationException,
			IllegalAccessException, ClassNotFoundException, SQLException {

		// Ustawienia dla polaczenia, sesja oraz storage polaczenia
		Properties properties = System.getProperties();
		Session session = Session.getDefaultInstance(properties);
		Store store = session.getStore("pop3");
		store.connect(host, user, password);

		// Otwieramy folder w trybie rw.
		Folder folder = store.getFolder("inbox");
		folder.open(Folder.READ_WRITE);

		// Pobieramy ostatnia wiadomosc
		Message message = folder.getMessage(folder.getMessageCount()
				- messageFromEnd);

		// Pobieramy zalacznik ostatniej wiadomosci.
		Multipart multipart = (Multipart) message.getContent();
		BodyPart bodyPart = multipart.getBodyPart(1);

		// Sprawdzamy czy parsowalismy juz tego emaila.
		// Jesli parsowalismy piszemy do log4j i konczymy dzialanie

		// Logujemy do log4j nazwe zalacznika
		logger.info("Pobieram zalacznik o nazwie: "
				+ multipart.getBodyPart(1).getFileName());

		// Sprawdzamy czy zalacznik zostal juz sparsowany.
		if(new DbManager().isAttachmentParsed(multipart.getBodyPart(1).getFileName())) {
			logger.warn("Zalacznik o nazwie: " + multipart.getBodyPart(1).getFileName() + " zostal juz sparsowany...");
			return "brak";
		}
		// Tworzymy Stringa z zalacznika
		InputStream stream = bodyPart.getInputStream();

		BufferedReader br = new BufferedReader(new InputStreamReader(stream, "CP1250"));

		String result = "";

		while (br.ready()) {
			String line = br.readLine();
			result = result + line;
		}

		// Zamykamy folder oraz polaczenie.
		folder.close(true);
		store.close();

		// Zwracamy Stringa
		return result;
	}

	public void sendEmailReport(ArrayList<Invoice> invoices)
			throws MessagingException {
		// pass @23kmycxU#
		
		// Logujemy ze wysylamy raport mailowy
		logger.info("Wysylam raport mailowy...");
		
		// Ustawiamy dostep do skrzynki pocztowej
		Properties props = new Properties();
		props.setProperty("mail.transport.protocol", "smtp");
		props.setProperty("mail.host", "smtp.");
		props.setProperty("mail.user", "");
		props.setProperty("mail.password", "");

		Session mailSession = Session.getDefaultInstance(props);
		Message message = new MimeMessage(mailSession);

		// Ustawiamy adresata oraz odbiorce.
		InternetAddress fromAddress = new InternetAddress("lmsimport@jpk.pl");
		InternetAddress toAddress = new InternetAddress("lmsraport@jpk.pl");

		message.setFrom(fromAddress);
		message.setRecipient(RecipientType.TO, toAddress);
		
		// Ustawiamy temat
		message.setSubject("[ LMSImport ] Raport dzialalnosci LMSImportera");

		// Iterujemy po wszystkich wplatach.
		// Dodajemy informacje o wplacie, oraz
		// dodajemy do wielkosc wplaty do sumy wplat
		Double summaryCash = 0.0D;
		String content = "";
		for (Invoice invoice : invoices) {
			content += invoice.toString() + "<br />";
			summaryCash += Double.parseDouble(invoice.getSum()
					.replace(',', '.'));
		}
		
		// Zaokraglamy do 2 miejsc po przecinku
		summaryCash *= 100;
		summaryCash = (double) Math.round(summaryCash);
		summaryCash /= 100;
		
		// Dodajemy informacje o sumie wplat
		content += "<br /><br />";
		content += "Suma wplat podczas tego importu: " + summaryCash + " zl";
		message.setContent(content, "text/html; charset=UTF-8");

		// Wysylamy wiadomosc.
		Transport.send(message);

	}
}
